import className from 'classnames'
import React from 'react'

let Item = React.createClass({
  propTypes: {
    renderLabel: React.PropTypes.func,
    onSelectItem: React.PropTypes.func
  },
  getDefaultProps () {
    return {
      renderLabel: item => item.label,
      onSelectItem: () => {},
      onMouseEnterItem: () => {}
    }
  },
  getInitialState () {
    return {
      expanded: true
    }
  },
  toggle (e) {
    e.preventDefault()
    this.setState({
      expanded: !this.state.expanded
    })
  },
  getSelectedColor () {
    let { item, isSelected } = this.props
    return isSelected(item)
    ? '#38A466'
    : ''
  },
  render () {
    let { item } = this.props
    return (
      <li
        id={`tree-item-${item.key}`}
        role='treeitem'
        aria-level={item.level}
        aria-expanded={this.state.expanded}
        style={item.style}>
        <div className='slds-tree__item restriction-tree__item'>
          <button
            aria-controls={`tree-item-${item.key}`}
            className={className('restriction-tree__item-toggle slds-button slds-m-right--x-small', {
              'slds-is-disabled': !item.hasChildren
            })}
            onClick={this.toggle}>
            <svg
              aria-hidden={!item.hasChildren}
              className='slds-button__icon slds-button__icon--small'>
              <use xlinkHref='/assets/icons/utility-sprite/svg/symbols.svg#chevronright' />
            </svg>
            <span className='slds-assistive-text'>{item.label}</span>
          </button>
          <div id={`tree-item-label-${item.key}`} className='slds-grow'>
            <div
              style={{color: this.getSelectedColor()}}
              onClick={(e) => this.props.onSelectItem(item, e)}
              onMouseEnter={(e) => this.props.onMouseEnterItem(item, e)}
              className={this.props.isSelected(item) ? 'is-selected' : null}>
              <span className='checked-mark'>
                <span />
              </span>
              {this.props.renderLabel(item)}
            </div>
          </div>
        </div>
        {item.hasChildren && this.state.expanded
          ? (
            <ul
              className={className({
                'slds-is-expanded': this.state.expanded,
                'slds-is-collapsed': !this.state.expanded
              })}
              role='group'
              aria-labelledby={`tree-item-label-${item.key}`}>
              {item.value.map((item) =>
                <Item
                  {...this.props}
                  key={item.key}
                  item={item} />
              )}
            </ul>
          )
          : null
        }
        <style jsx>{`
          .restriction-tree__item {
            display: flex;
            line-height: 1.5rem;
            padding: 0;
          }
          .restriction-tree__item:hover {
            background: transparent;
          }
          .restriction-tree__item-toggle {
            display: inline-flex;
            line-height: initial;
            border-radius: 0.125rem;
            align-self: flex-start;
            position: relative;
            top: 0.35rem;
          }
          .checked-mark {
            display: inline-block;
            position: relative;
            width: 0.5rem;
            height: 0.5rem;
            border-radius: 50%;
            background: #EFF3F8;
            border: 1px solid currentColor;
            margin-right: 0.25rem;
            vertical-align: middle;
          }
          .checked-mark > span {
            display: inline-block;
            position: absolute;
            top: 50%;
            left: 50%;
            transform: translate3d(-50%,-50%,0);
            width: 0.25rem;
            height: 0.25rem;
            border-radius: 50%;
            background: currentColor;
            opacity: 0;
          }
          .is-selected .checked-mark > span {
            opacity: 1;
          }
        `}</style>
      </li>

    )
  }
})

export default props =>
  !props.item ? null : <ul className='slds-tree restriction-tree' role='tree'>
    <Item {...props} key={props.item.key} />
  </ul>
