import { constant, identity } from 'core.lambda'
import Either from 'data.either'
import React from 'react'
import uid from 'uid-safe'

const { applyModifiers } = require('@salesforce-ux/design-system-markup/browser')

const is = type => o =>
  typeof o === type ? Either.Right(o) : Either.Left(o)

export default React.createClass({
  getDefaultProps () {
    return {
      markup: '',
      styles: Date.now(),
      width: '100%'
    }
  },
  getInitialState () {
    return {
      id: uid.sync(18),
      height: 100
    }
  },
  componentDidMount () {
    this.updateMarkup(this.props.markup)
    this.updateStyles(this.props.styles)
    window.addEventListener('message', message =>
      Either
        .fromNullable(message.data)
        .chain(is('object'))
        .chain(action =>
          is('string')(action.type)
            .map(constant(action))
        )
        .chain(action =>
          is('object')(action.payload)
            .map(constant(action))
        )
        .chain(action =>
          action.payload.id === this.state.id
            ? Either.Right(action) : Either.Left(action)
        )
        .fold(identity, action => {
          switch (action.type) {
            case 'iframe:height':
              this.setState({
                height: action.payload.height
              })
              break
            case 'iframe:applyModifiers':
              applyModifiers(
                this.props.modifiers,
                this.iframe.contentDocument.body
              )
              break
          }
        })
    )
  },
  componentDidUpdate (prevProps) {
    if (this.props.markup !== prevProps.markup) {
      this.updateMarkup(this.props.markup)
    }
    if (this.props.styles !== prevProps.styles) {
      this.updateStyles(this.props.styles)
    }
    if (this.props.modifiers !== prevProps.modifiers) {
      this.updateMarkup(this.props.markup)
    }
  },
  updateMarkup (markup) {
    this.postMessage('markup', {
      markup
    })
  },
  updateStyles (styles) {
    this.postMessage('styles', {
      styles
    })
  },
  getContentOrigin () {
    let { protocol, host } = window.location
    return `${protocol}//${host}`
  },
  postMessage (type, payload) {
    const message = {
      type,
      payload: Object.assign({}, payload, {
        id: this.state.id
      })
    }
    this.iframe.contentWindow.postMessage(message, this.getContentOrigin())
  },
  render () {
    return <div>
      <iframe
        src='/__IFRAME__'
        ref={node => {
          this.iframe = node
        }}
        width={this.props.width}
        height={this.state.height}
        onLoad={() =>
          this.updateMarkup(this.props.markup)
        } />
      <style jsx>{`
        iframe {
          margin: 0;
          border: none;
        }
      `}</style>
    </div>
  }
})
