import Either from 'data.either'
import { List } from 'immutable'
import React from 'react'

import { cata } from '../model'
import { dispatch } from '../shared/io'
import HierarchyTree from './HierarchyTree'

const mungeIt = (comment, level = 0) => {
  const label = comment.get('id')
  const children = comment.get('restrictees') || List()
  return {
    comment,
    level,
    label,
    key: label,
    value: children.map(x => mungeIt(x, level + 1)),
    numChildren: children.count(),
    hasChildren: children.count() > 0
  }
}

const toggleItem = (modifiers, modifier) =>
  Either
    .of(modifiers.findIndex(m => m.get('id') === modifier.get('id')))
    .map(index =>
      index >= 0
        ? modifiers.delete(index)
        : modifiers.push(modifier)
    )
    .get()

export default props =>
  <div>
    <HierarchyTree
      {...props}
      style={props.style}
      item={props.hierarchy ? mungeIt(props.hierarchy) : null}
      isSelected={item =>
        props.modifiers.map(x => x.get('id')).includes(item.comment.get('id'))
      }
      onSelectItem={item =>
        cata(item.comment, {
          modifier: mod =>
            dispatch({
              type: 'modifierSelected',
              payload: {
                modifiers: toggleItem(props.modifiers, mod)
              }
            })
        })
      }
      renderLabel={item => {
        let renderBadge = () =>
          cata(item.comment, {
            modifier: m =>
              !m.has('group')
              ? null
              : <span key='badge' className='modifier-group-badge slds-m-left--xx-small'>{m.get('group')}</span>
          })
        let renderLabel = () =>
          <span key='label'>{item.label}</span>
        let renderIcon = () =>
          <span key='icon' className='selector-type-badge'>
            {cata(item.comment, {
              component: () => 'C ',
              variant: () => 'V ',
              modifier: () => 'M ',
              element: () => 'E '
            })}
          </span>
        return [
          renderIcon(),
          renderLabel(),
          renderBadge()
        ]
      }}
    />
    <style jsx>{`
      .selector-type-badge {
        display: inline-block;
        width: 1rem;
        height: 1rem;
        font-size: .5rem;
        background: #DBE1EA;
        box-shadow: rgba(0,0,0,.15) 0 0 0 1px inset;
        border-radius: 50%;
        text-align: center;
        line-height: 1rem;
        vertical-align: middle;
        margin-right: 0.125rem;
      }
      .modifier-group-badge {
        display: inline-block;
        font-size: .75rem;
        line-height: 1.2;
        background: #DBE1EA;
        box-shadow: rgba(0,0,0,.15) 0 0 0 1px inset;
        border-radius: 0.125rem;
        padding: 0 2px;
        opacity: .8;
        transition: all .35 linear;
      }
    `}</style>
  </div>
