import marked from 'marked'
import { Map, Set } from 'immutable'
import React from 'react'

import Select from './Select'
import SelectorTree from './SelectorTree'

import { dispatch } from '../shared/io'

const keyIn = (...keys) => {
  const keySet = Set(keys)
  return (v, k) => keySet.has(k)
}

export default React.createClass({
  getInitialState () {
    return {}
  },
  updateComponent (e) {
    dispatch({
      type: 'componentSelected',
      payload: { component: e.target.value }
    })
  },
  updateVariant (e) {
    dispatch({
      type: 'variantSelected',
      payload: { variant: e.target.value }
    })
  },
  render () {
    const { server, style } = this.props
    return (
      <div
        className='app-stage-left slds-grid slds-grid--vertical'
        style={style}>
        <div className='picker'>
          <span className='logo slds-size--1-of-4' title='Salesforce Logo'>
            <svg className='slds-icon slds-icon--large' aria-hidden='true'>
              <use xlinkHref='/assets/icons/utility-sprite/svg/symbols.svg#salesforce1' />
            </svg>
          </span>
          <div className='slds-grid slds-col--bump-left slds-size--3-of-4'>
            <div className='slds-form-element slds-size--1-of-2 slds-p-horizontal--x-small'>
              <label className='slds-form-element__label'>Component</label>
              <div className='slds-form-element__control'>
                <Select
                  onChange={this.updateComponent}
                  options={server.get('components')}
                  value={server.get('component').get('id')} />
              </div>
            </div>
            <div className='slds-form-element slds-size--1-of-2 slds-p-horizontal--x-small'>
              <label className='slds-form-element__label'>Variant</label>
              <div className='slds-form-element__control'>
                <Select
                  onChange={this.updateVariant}
                  options={server.get('variants')}
                  value={(server.get('variant') || Map()).get('id')} />
              </div>
            </div>
          </div>
        </div>
        <div className='metadata slds-text-longform'>
          <h2 className='slds-text-heading--label'>Component Metadata</h2>
          <p dangerouslySetInnerHTML={{
            __html: marked(server.getIn(['component', 'description'], ''))
          }} />
          {server.getIn(['component', 'annotations'])
            .filterNot(keyIn('base'))
            .map(v => typeof v === 'string' ? marked(v) : v)
            .map((v, k) =>
              <dl key={k}>
                <dt><strong>{k}</strong></dt>
                <dd dangerouslySetInnerHTML={{ __html: v }} />
              </dl>
            )
            .valueSeq()
            .toArray()}
        </div>
        <div className='slds-grid slds-grid--vertical restrictions'>
          <h2 className='slds-text-heading--label slds-m-bottom--small'>Restriction Tree</h2>
          <SelectorTree
            hierarchy={server.get('hierarchy')}
            modifiers={server.get('modifiers')}
            onMouseEnterItem={item => {
              this.setState({
                selectedComment: item.comment
              })
            }} />
        </div>
        <div className='metadata-hover slds-text-longform'>
          <h2 className='slds-text-heading--label'>Classname Documentation</h2>
          {!this.state.selectedComment
            ? null
            : <p dangerouslySetInnerHTML={{
              __html: marked(this.state.selectedComment.get('description', ''))
            }} />
          }
          {!this.state.selectedComment
            ? null
            : this.state.selectedComment.get('annotations')
              .filterNot(keyIn('variant', 'modifier'))
              .map(v => typeof v === 'string' ? marked(v) : v)
              .map((v, k) =>
                <dl key={k}>
                  <dt><strong>{k}</strong></dt>
                  <dd dangerouslySetInnerHTML={{ __html: v }} />
                </dl>
              )
              .valueSeq()
              .toArray()
          }
        </div>
        <style jsx>{`
          .app-stage-left {
            background-color: #F4F6F9;
            border-right: 1px solid #D8DDE6;
          }
          .picker {
            display:flex;
            padding: 0.5rem 0.5rem 0.5rem 2rem;
          }
          .logo {
            color: #009DDC;
            align-self: center;
          }
          .logo svg {
            fill: currentColor;
          }
          .metadata {
            flex: 1 0 auto;
            height: 0;
            padding: 2rem;
            overflow-y: auto;
            overflow-x: hidden;
          }
          .restrictions {
            flex: 3 0 auto;
            height: 0;
            padding: 2rem;
            overflow-y: auto;
            overflow-x: hidden;
          }
          .metadata-hover {
            flex: 1 0 auto;
            height: 0;
            padding: 2rem;
            overflow-y: auto;
            overflow-x: hidden;
          }
          .restriction-tree {
            line-height: 1.5rem;
          }
        `}</style>
      </div>
    )
  }
})
